Imports Microsoft.DirectX
Imports Microsoft

Public Class Form1

    Private mobjDevice As Direct3D.Device
    Private mobjPresent As Direct3D.PresentParameters
    Private mobjEffect As Direct3D.Effect
    Private mobjMesh As Direct3D.Mesh
    Private mobjRotation As New Vector3(0, 0, 0)


    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        tmrRender.Stop()
        mobjPresent.Windowed = False
        mobjMesh.Dispose()
        mobjMesh = Nothing
        mobjEffect.Dispose()
        mobjEffect = Nothing
        mobjDevice.Dispose()
        mobjDevice = Nothing
    End Sub

    Private Sub Form1_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        Select Case e.KeyCode
            Case Keys.Up
                mobjEffect.SetValue("fMultiplier", mobjEffect.GetValueFloat("fMultiplier") + 0.1F)

            Case Keys.Down
                mobjEffect.SetValue("fMultiplier", mobjEffect.GetValueFloat("fMultiplier") - 0.1F)

            Case Keys.Escape
                Me.Close()

        End Select
    End Sub

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' setup present params
        mobjPresent = New Direct3D.PresentParameters
        mobjPresent.AutoDepthStencilFormat = Direct3D.DepthFormat.D16
        mobjPresent.EnableAutoDepthStencil = True
        mobjPresent.SwapEffect = Direct3D.SwapEffect.Discard
        mobjPresent.Windowed = True
        mobjPresent.PresentationInterval = Direct3D.PresentInterval.Immediate

        ' create gfx device
        mobjDevice = New Direct3D.Device(Direct3D.Manager.Adapters.Default.Adapter, Direct3D.DeviceType.Hardware, Me, Direct3D.CreateFlags.HardwareVertexProcessing, mobjPresent)

        ' load effect file
        mobjEffect = Direct3D.Effect.FromFile(mobjDevice, "GreyScale.fx", Nothing, Nothing, Direct3D.ShaderFlags.None, Nothing)

        ' load mesh
        mobjMesh = Direct3D.Mesh.FromFile("Sphere.x", Direct3D.MeshFlags.Managed, mobjDevice)

        ' load texture and set texture that the effect will use
        Dim Tex As Direct3D.Texture = Nothing
        Tex = Direct3D.TextureLoader.FromFile(mobjDevice, "Fieldstone.png")
        mobjEffect.SetValue("base_Tex", Tex)

        tmrRender.Start()
    End Sub

    Private Sub tmrRender_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrRender.Tick
        If Me.WindowState = FormWindowState.Minimized Then Return

        ' calc aspect of form
        Dim aspect As Single
        aspect = CSng(Me.ClientSize.Width / Me.ClientSize.Height)

        ' clear and setup scene
        mobjDevice.Clear(Direct3D.ClearFlags.Target Or Direct3D.ClearFlags.ZBuffer, Color.Black, 1, 0)
        mobjDevice.BeginScene()

        ' setup rotation
        Dim R As Vector3
        R = New Vector3(Direct3D.Geometry.DegreeToRadian(mobjRotation.X), _
                        Direct3D.Geometry.DegreeToRadian(mobjRotation.Y), _
                        Direct3D.Geometry.DegreeToRadian(mobjRotation.Z))

        ' setup the world/view/projection matrix
        Dim Mat As Matrix = Matrix.Identity
        Mat.Multiply(Matrix.RotationYawPitchRoll(R.X, R.Y, R.Z))
        Mat.Multiply(Matrix.LookAtLH(New Vector3(0, 0, -100), New Vector3(0, 0, 0), New Vector3(0, -1, 0)))
        Mat.Multiply(Matrix.PerspectiveFovLH(Math.PI / 4.0F, aspect, 1, 10000))

        ' set the effects WVP matrix and rendering Technique to use
        mobjEffect.SetValue("matWorldViewProjection", Mat)
        mobjEffect.Technique = "Textured"

        ' now render mesh using effect
        Dim passes As Integer
        ' begin rendering the Technique and get the number of rendering passes thre are in the Technique
        passes = mobjEffect.Begin(Direct3D.FX.None)
        For idx As Integer = 0 To passes - 1
            ' begin rendering pass
            mobjEffect.BeginPass(idx)

            ' render the geometry
            mobjMesh.DrawSubset(0)

            ' done with pass
            mobjEffect.EndPass()
        Next
        ' done rendering Technique
        mobjEffect.End()

        ' present scene
        mobjDevice.EndScene()
        mobjDevice.Present()
    End Sub

    Private Sub Form1_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        Select Case e.Button
            Case Windows.Forms.MouseButtons.Left
                mobjRotation.X = e.X

            Case Windows.Forms.MouseButtons.Middle
                mobjRotation.Y = e.X

            Case Windows.Forms.MouseButtons.Right
                mobjRotation.Z = e.X

        End Select
    End Sub
End Class
